<?php

/* NoPro sql-lib
 * some handy methods extending mysqli-object
 */

require_once('nopro_lib.php');
// nopro_lib.php must be included here

class NopSQLO extends mysqli {
  
  /**
   * 2015-10-10 v0.1 based on nopro_sqli.php
   * 2016-05-06 q2arr() now uses mysqli::fetch_all() instead of a loop
   */

  public function query($q, $verb=false) {
    if($verb)
      echo $q.LF;
    $rv = parent::query($q);
    if($rv === false) {
      echo $this->error .LF;
    }
    return $rv;
  }
  
  public function store($tbl, $arr, $idkey = null, $verbose = false, $keys = null) {
    /* the presence of $idkey, signals UPDATE instead of INSERT
     * $verbose means to print the query
     * $keys for non-associative $arr, or to override $k from $arr
     */
    $query = $idkey ? "UPDATE $tbl SET" : "INSERT INTO $tbl SET";
    foreach($arr as $k => $val) {
      $key = $keys ? $keys[$k] : $k;
      if($key == $idkey || (is_array($idkey) && in_array($key, $idkey)))
	continue;
      if(is_numeric($val))
	$query .= " `$key` = $val,";
      else if($val === null)
	$query .= " `$key` = NULL,";
      else
	$query .= " `$key` = '$val',";
    }
    $query = rtrim($query, ',');
    
    if($idkey) {
      if(is_array($idkey)) {
	$query .= 'WHERE';
	foreach($idkey as $k) {
	  $query .= " `$k` = '$arr[$k]' AND";
	}
	$query = substr($query, 0, strlen($query) - 4);
      } else {
	$query .= sprintf(" WHERE `$idkey` = '%s'", $arr[$idkey]);
      }
    }
    
    if($verbose)
      echo $query ."\n";
    #die($query);
    
    $this->query($query) OR die("store(): ". $this->error);
    #return ($idkey && !is_array($idkey)) ? $arr[$idkey] : $this->insert_id;
  }
  
  public function q2arr($query, $assoc=true) {
    // fill array with sql-results
    $res = $this->query($query) OR die("query(): ".$this->error);
    if(1): // use fetch_all

    $arr = $res->fetch_all($assoc ? MYSQLI_ASSOC : MYSQLI_NUM); 

    else: // use loop

    $arr = array();
    if($assoc)
      while($row = $res->fetch_assoc()) { array_push($arr, $row); }
    else
      while($row = $res->fetch_row()) { array_push($arr, $row); }
    
    endif; // use loop
    
    $res->close();
    return $arr;
  }
  
  public function q2arr1d($query) {
    // fill 1d-array with sql-results
    $arr = array();
    $res = $this->query($query) OR die("query(): ".$this->error);
    while($row = $res->fetch_row()) {
      array_push($arr, $row[0]);
    }
    $res->close();
    return $arr;
  }
  public function tables() { return $this->q2arr1d('SHOW TABLES'); }
  
  public function coll($query, $dim = 1) {
    // return array ('collection') as result of query
    return $dim > 1 ? $this->q2arr( $query) : $this->q2arr1d($query);
  }
  
  public function roster($tbl, &$prim = false) {
    // return array of fname => ftyype
    $ret = array();
    $xpl = $this->q2arr("EXPLAIN $tbl");   // DESCRIBE table
    foreach($xpl as $arr) {
      $ret[$arr['Field']] = $arr['Type'];
    }
    if($prim !== false) $prim = sql_find_pri($xpl);
    return $ret;
  }
  
  public function q2csv($q, $out = '/dev/stdout') {
    // output query-result as csv
    $res = $this->query($q) OR die("query(): ".$this->error);
    if(!is_bool($res) && $res->num_rows > 0) {
      $fh = fopen($out, 'w') OR die("fopen($out)");
      $row = $res->fetch_assoc();
      fputcsv($fh, array_keys($row));
      fputcsv($fh, array_values($row));
      # while($row = $res->fetch_assoc()) {
      while($row = $res->fetch_row()) {
	fputcsv($fh, $row);
      }
      fclose($fh);  
      $res->close();
    }
  }
  
  public function q2out($query) {
    // echo sql-results
    $res = $this->query($query) OR die("query(): ".$this->error);
    $row = $res->fetch_assoc();
    if($row)
      foreach(array_keys($row) as $k)
	echo "$k\t";
    echo LF;
    while($row) {
      foreach($row as $v)
	echo "$v\t";
      echo LF;
      $row = $res->fetch_row();
    }
    $res->close();
  }
  
  public function setXchecks($on, $verb=false) {
    // switch foreign key-checks 
    if(!$on)
      $this->query("SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0", $verb);
    else
      $this->query("SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS", $verb);
  }
  
}
