<?php
# wget
/***
 * 2015-09-23 npwg_split(): fixed skip 100
 * 2016-02-21 added npwg_cache()
 * 2016-02-23 added npwg_echo() and verbosity
 * 2016-03-06 added npwg_2refresh
 * 2016-04-02 added npwg_cachedel(), changed microtimer: npw_get() itself is timed now
 * 2016-04-04 added npwg_fetch() and npwg_look()
 * 2016-04-05 added npwg_hdr_stati()
 * 2016-04-30 added npwg_check_url()
 * 2016-05-02 npwg_split() recognizes proxies closing
 */

  if(!defined('CR')) define('CR', "\r");
  if(!defined('LF')) define('LF', "\n");
  if(!defined('CRLF')) define('CRLF', CR.LF);
  if(!defined('Cache')) {
    if(is_dir('/dev/shm')) define('Cache', '/dev/shm');
    else if(getenv('TEMP')) define('Cache', getenv('TEMP'));
    else if(getenv('TMP')) define('Cache', getenv('TMP'));
    else define('Cache', '/tmp');
  }
  $npwg_verb = 1; // verbosity-lvl for npwg_echo()
  function npwg_echo($msg, $verb=null) {
    // echo msg, unless verbosity is zero
    global $npwg_verb;
    if($verb !== null) $npwg_verb = $verb;
    if($npwg_verb)
      echo $msg;
  }

  function npwg_hdr_skip($resp, $p1) {
    // append http-header after these status-msgs
    $skips = array(
		   'HTTP/1.1 100 Continue',
		   'HTTP/1.0 200 Connection established',
		   'HTTP/1.1 200 Connection established',
		   );
    foreach($skips as $skip)
      if(strpos($resp, $skip, $p1) === $p1)
	return true;
    return false;
  }

  function npwg_split($resp) {
    // split http-header and -contents (honouring HTTP 100 replies)
    $p1 = 0;
    while(npwg_hdr_skip($resp, $p1))
      $p1 = 4 + strpos($resp, CRLF.CRLF, $p1);
    $p1 = strpos($resp, CRLF.CRLF, $p1);
    
    $smsg = "Proxy-Connection: close";
    $prox = stripos($resp, $smsg);  
    if($prox && $prox < $p1) {
      $p1 = strpos($resp, CRLF.CRLF, $p1+4);
    }
    
    $hdr = substr($resp, 0, $p1+2);
    $cnt = substr($resp, $p1+4);
    return array($hdr, $cnt);
  }

  function npwg_get($url, $copt=null, $post = NULL, $cookies = null, $body=TRUE) {

    if(strpos($url, '://') === false) $url = "http://$url";
    npwg_echo(empty($post) ? $body ? 'GET' : 'HEAD' : 'POST');
    npwg_echo(" $url" .LF);

    $ch = curl_init();
    
    $header[] = "Accept-Encoding: gzip,deflate";
#    curl_setopt($ch, CURLOPT_HTTPHEADER, $header );
    curl_setopt($ch, CURLOPT_ENCODING , "gzip,deflate");
  
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_HEADER, true);	  // 1 = true
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_NOBODY, !$body);
    //curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    //curl_setopt($ch, CURLOPT_AUTORFERER, true);
    //curl_setopt($ch, CURLOPT_MAXREDIRS, 2);
    //curl_setopt($ch, CURLOPT_COOKIEFILE, '');
    //curl_setopt($ch, CURLOPT_COOKIEJAR, '');
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);  // default is 0 (for never)
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);	  // default is 0 (for never)
    
    if(!empty($cookies)) curl_setopt($ch, CURLOPT_COOKIE, $cookies);
    if(!empty($copt) && is_array($copt))
      foreach($copt as $k => $v)
	curl_setopt($ch, $k , $v);
    if(!empty($post)) curl_setopt($ch, CURLOPT_POSTFIELDS, $post);

    $tim0 = microtime(true);
    $resp = curl_exec($ch);	// grab Response as string
    curl_close($ch);		// close cURL resource, and free up system resources
    $tim1 = microtime(true);
    npwg_echo(sprintf("-- time: %6.3f".LF, $tim1 - $tim0));
    
    return $resp;
  }
  function npwg_post($url, $copt, $post, $cookie=null) {
    return npwg_get($url, $copt, $post, $cookie);
  }
  function npwg_head($url, $copt=null, $post = NULL, $cookie = null) {
    return npwg_get($url, $copt, $post, $cookie, FALSE);
  }
  function npwg_info($url, $copt=null, $post = NULL, $cookie = null) {
    $http = npwg_get($url, $copt, $post, $cookie);
    return substr($http, 0, strpos($http, CRLF));
  }

  function npwg_showget($url, $copt=null, $post = NULL, $cookie = null, $body=TRUE) {
    $resp = npwg_get($url, $copt, $post, $cookie, $body);
    list($hdr, $cnt) = npwg_split($resp);
    echo $hdr ."================".CRLF;
    if(isset($_SERVER["HTTP_HOST"]))
      echo str_replace('<', '&lt;', $cnt);
    else
      echo $cnt;
    return array($hdr, $cnt);
  }

  function npwg_showhead($url, $copt=null, $post = NULL, $cookie = null, $body=FALSE) {
    $resp = npwg_get($url, $copt, $post, $cookie, $body);
    list($hdr, $cnt) = npwg_split($resp);
    echo $hdr;
    if($body)
      echo "Length=".strlen($cnt);
    echo CRLF."================".CRLF;
    return array($hdr, $cnt);
  }

  function npwg_parse_hdr($hdr, $offset = 0) {
    if(!$hdr) return (object)array('stat' => 0);
    $head = new stdclass;
    foreach(explode(CRLF, $hdr) as $line) {
      if(!$line) continue;
      if(strpos($line, ': ') === false) {
	npwg_echo($line.LF);
	list($head->prot, $head->stat, $head->smsg) = explode(' ', $line, 3);
      } else {
	list($k, $v) = explode(': ', $line, 2);
	$head->$k = $v;
      }
    }
    return $head;
  }

  function npwg_get_cookie($hdr) {
    $cookie = '';
    $cc = 'Set-Cookie: ';
    $p1 = strpos($hdr, $cc); if($p1 === false) return $p1;
    while($p1) {
      $p1 += strlen($cc);
      $l = strcspn($hdr, " ;\r\n", $p1);
      $tmp = substr($hdr, $p1, $l);
      /**
      $p1 = strpos($hdr, 'expires=', $p1);
      $l = strcspn($hdr, ";\r\n", $p1);
      $exp = substr($hdr, $p1+8, $l-8);
      echo "exp: $exp";
      dbg(strtotime($exp));
      #dbg(date_parse($exp));
      /**/
      $cookie = $tmp;
      $p1 = strpos($hdr, $cc, $p1);
    }
    return $cookie;
  }

  function npwg_delay($t = 5000) {
    if($t) npwg_echo("delay($t msec) ...".LF);
    usleep($t*1000);
  }

  function npwg_2refresh($cache, $refresh) {
    // decide if cache is out-dated (elder than refresh minutes)
    if(!is_int($refresh)) return true;
    $t0 = filemtime($cache);
    $t1 = time() - $t0;
    return $t1 > 60*$refresh;
  }
  function npwg_cache($url, $copt, $refresh=false, $cache=null) {
    // if(!$cache) $cache = Cache."/".md5($url);
    if(!$cache) $cache = Cache."/".crc32($url);
    if(file_exists($cache."-cont") && (!$refresh || !npwg_2refresh($cache."-url", $refresh))) {
      npwg_echo("CACHED $url".LF);
      $cont = file_get_contents($cache."-cont");
      $head = file_get_contents($cache."-head");
    } else {
      list($head, $cont) = npwg_split(npwg_get($url, $copt));
      file_put_contents($cache."-cont", $cont);
      file_put_contents($cache."-head", $head);
      file_put_contents($cache."-url", $url);
    }
    return array($head, $cont);
  }

  function npwg_cachedel($url, $cache=null) {
    // delete a single cache entry
    // if(!$cache) $cache = Cache."/".md5($url);
    if(!$cache) $cache = Cache."/".crc32($url);

    npwg_echo("DEL CACHED $url".LF);
    @unlink($cache."-cont");
    @unlink($cache."-head");
    @unlink($cache."-url");
  }

  function npwg_fetch($url, $copt=null, $post = NULL, $cookies = null) {
    // conveniently return cont and parsed head
    list($hdr, $cnt) = npwg_split(npwg_get($url, $copt, $post, $cookies));
    return array(npwg_parse_hdr($hdr), $cnt);
  }

  function npwg_look($url, $copt=null, $post = NULL, $cookies = null) {
    // send HEAD-request and conveniently return parsed head
    return npwg_parse_hdr(npwg_head($url, $copt, $post, $cookies));
  }

  function npwg_hdr_stati($hdr) {
    // filters all but status-lines from header
    $stati = '';
    foreach(explode(CRLF, $hdr) as $l)
      if(strpos($l, 'HTTP/') === 0)
	$stati .= $l.CRLF;
    return $stati;
  }

  function npwg_check_url($url, $copt=null) {
    // send HEAD-request and computes reply
    $head = npwg_look($url, $copt);
    if(!$head->stat) return false;
    #npwg_echo(sprintf("%s %s".LF, $head->stat, $head->smsg));   // done already in npwg_parse()
    switch($head->stat) {
    case 200: $found = $url; break;
    case 301:
    case 302: $found = $head->Location; break;
    default:
       $found = $head->stat; break;
    }
    return $found;
  }