<?php
# html-parser

/**
 * (c) NoPro 2016
 * version 0.1 2016-04-02
 *
 * based on ml_parser
 * (c) NoPro 2015-2016
 * version 0.1 2015-10-06
 */

require_once('ml_parser.php');

class html_snippet extends ml_snippet {

  // snippet-types
  const SnipTABLE  = 2;
  const SnipFORM   = 4;
  const SnipSCRIPT = 8;

  public function __construct(&$doc, $p1, $tag=false, $type=0) {
    parent::__construct($doc, $p1, $tag, $type);
    
    switch($this->get_tag()) {
    case 'SCRIPT': $this->type = self::SnipSCRIPT; break;
    case 'TABLE':  $this->type = self::SnipTABLE;  break;
    case 'FORM':   $this->type = self::SnipFORM;   break;
    }
    
  }

  public function is_script() { return $this->type & self::SnipSCRIPT; }
  public function is_table()  { return $this->type & self::SnipTABLE; }
  public function is_form()   { return $this->type & self::SnipFORM; }
  public function is_hole()   { return $this->type & (self::SnipREMARK | self::SnipSCRIPT); }
  
  public function get_form() {
    // return post-array with all set values of INPUT, SELECT and TEXTAREA objects of a form-snip
    if(!$this->is_form())
      return false;
    $post = array();
    // examine INPUTs
    foreach($this->get_tags('INPUT') as $i) {
      $attrs = $i->get_attrs();
      if(isset($attrs['name']) && empty($attrs['disabled'])) {
	$itype = isset($attrs['type']) ? $attrs['type'] : '';
	switch($itype) {
	case 'button':
	case 'reset':
	//case 'submit':
	  break;
	  
	case 'checkbox':
	case 'radio':
	  if(!empty($attrs['checked']))
	    $post[$attrs['name']] = $attrs['checked'];
	  break;
	  
	default:
	  $post[$attrs['name']] = isset($attrs['value']) ? $attrs['value'] : '';
	}
      }
    }
    // examine SELECTs
    foreach($this->get_tags('SELECT') as $i) {
      ;
    }
    // handle TEXTAREAs
    foreach($this->get_tags('TEXTAREA') as $i) {
      $attrs = $i->get_attrs();
      if(isset($attrs['name']))
	$post[$attrs['name']] = $i->cont();
    }
    return $post;
  }
  
} // EOclass html_snippet


class html_parser extends ml_parser {

  /**
  public function __destruct() {
    dbg($this->info());
    foreach($this->snips as $s)
      echo $s.LF;
  }
  /**/

  public function __construct(&$doc, $file=false, $init=true) {
    $p1 = strpos($doc, '<');
    if($p1) echo "doc begins @ $p1".LF;
    if($p1 === false) return;
    if(stripos($doc, '<html', $p1) == $p1
       || strpos($doc, '<!--[', $p1) == $p1
       ) $this->doctype = 'HTML';
    parent::__construct(substr($doc, $p1), $file, $init);
  }
    
  protected function add_snippet($p1, $tag, $type=0) {
    $snip = new html_snippet($this->doc, $p1, $tag, $type);
    $this->snips[] = $snip;
    return $snip;
  }

  public function get_forms() {
    // return all html-forms of a doc
    if($this->doctype == 'HTML')
      return $this->get_tags('FORM');
    return false;
  }

} // EOclass html_parser
