<?php
# abstract ml-parser

/**
 * based on ml_parser (parser.php)
 * (c) NoPro 2015-2016
 * version 0.1 2015-10-06
 *
 * (c) NoPro 2016
 * version 0.1 2016-04-02
 */

require_once('nopro_lib/nopro_lib.php');
define('DEBUG', 0); // 0,1,2

class ml_snippet {

  // snippet-types
  const SnipREMARK = 1;
  
  protected
    $doc = null,	// reference to doc
    $p1, $p2,		// pointer into doc for begin and end
    $cp1, $cp2, 	// pointer into doc for contents-begin and -end
    $tag,		// ml-tag of snippet
    $type,		// snippet-type
    $len = 0,		// snippet length 
    $clen = 0,		// content length
    $mom = null,	// super-layer snip
    $kids = array(),	// snips in sub-layer 1
    $allkids = array(),	// snips in all sub-layers
    $EOprots;

  public function __construct(&$doc, $p1, $tag=false, $type=0) {
    $this->doc  = &$doc;
    $this->p1   = $p1;
    $this->type = $type;
    
    $p = ml_parser::eo_tag($this->doc, $this->p1);
    if($p !== false)
      $this->cp1 = $p + 1;
    
    if($tag === true) {
      $p = strcspn($this->doc, ' />', $this->p1);
      if($p !== false)
	$this->set_tag(substr($this->doc, $this->p1 + 1, $p - 1));
    } else if($tag) {
      $this->set_tag($tag);
    }
  }
  
  public function len() {return $this->len; }
  public function clen() {return $this->clen; }
  public function snip() { return $this->len ? substr($this->doc, $this->p1, $this->len) : false; }
  public function cont() { return $this->clen ? substr($this->doc, $this->cp1, $this->clen) : false; }
  public function get_tag() { return $this->tag; }
  public function get_beg() { return $this->p1; }
  public function get_end() { return $this->p2; }
  public function get_cbeg() { return $this->cp1; }
  public function get_cend() { return $this->cp2; }
  public function set_tag($tag) { $this->tag = strtoupper($tag); }
  public function get_mom() { return $this->mom; }
  public function set_mom($snip) { $this->mom = $snip; }
  public function add_kid($snip) { $this->kids[] = $snip; }
  public function get_kids() { return $this->kids; }
  public function count_kids() { return count($this->kids); }
  public function get_taglen() { return  ml_parser::eo_tag($this->doc, $this->p1) - $this->p1 + 1; }
  public function get_fulltag() { return substr($this->doc, $this->p1, $this->get_taglen()); }
  public function get_attrs() {
    // return array of attributes
    $tag = str_replace("'", '"', $this->get_fulltag()); 	// reduced strictness for single-quotes in html
    $p1 = strpos($tag, " "); if($p1 === false) return $p1;
    $attrs = array();
    while($p1) {
      $p2 = strpos($tag, "=", $p1); if($p2 === false) break;
      $k = trim(substr($tag, $p1+1, $p2-$p1-1));
      $p2 = strpos($tag, '"', $p2); if($p2 === false) break;
      $p3 = strpos($tag, '"', $p2+1); if($p3 === false) break;
      $v = substr($tag, $p2+1, $p3-$p2-1);
      $attrs[$k] = $v;
      $p1 = strpos($tag, " ", $p3);
    }
    return $attrs;
  }
  public function get_attr($attr) {
    // return value of attribute
    $tag = str_replace("'", '"', $this->get_fulltag()); 	// reduced strictness for single-quotes in html
    $p1 = strpos($tag, " $attr"); if($p1 === false) return '';
    $p1 = strpos($tag, '"', $p1); if($p1 === false) return '';
    $p2 = strpos($tag, '"', $p1+1); if($p2 === false) return '';
    return substr($tag, $p1+1, $p2-$p1-1);
  }
  public function set_end($p2, $empty=false) {
    // set end-position for snip
    if(!$p2) return false;
    $this->p2 = $p2;
    $this->len = 1 + $this->p2 - $this->p1;
    if($empty)
      $this->cp1 = $this->cp2 = $this->clen = 0;
    else {
      $this->cp2 = ml_parser::bo_tag($this->doc, $this->p2);
      $this->clen = $this->cp2 - $this->cp1;
    }
  }
  public function is_cont($doctype)  {
    // is snippet a single container-tag
    switch($doctype) {
    case 'XML': return true;	// XML doesnt have empty containers
    case 'HTML': 
      $singles = array('LINK', 'META', 'BR', 'HR', 'IMG', 'INPUT');
      return !in_array($this->get_tag(), $singles);
    default:
      echo "unknown doctype $doctype".LF;
    }
  }
  
  public function is_rem()    { return $this->type & self::SnipREMARK; }
  public function is_hole()   { return $this->type & self::SnipREMARK; }
  
  public function tree() {
    // lists whole tree as breadcrumbs
    $tree = $this->get_tag();
    $snip = $this;
    while($snip->mom) {
      $snip = $snip->mom;
      $tree = $snip->get_tag() ."->$tree";
    }
    return strtolower($tree);
  }
  
  public function info() {
    // put together some snip-info
    $info = new stdclass;
    $info->tag = $this->get_tag();
    $info->beg = $this->get_beg();
    $info->len = $this->len;
    $info->clen = $this->clen;
    $info->attr = $this->get_attrs();
    $mom = $this->get_mom();
    if($mom) {
      $info->momtag = $mom->get_tag();
      $info->mombeg = $mom->get_beg();
    }
    return $info;
  }

  protected function fill_allkids($kids) {
    // recursive fill array
    foreach($kids as $s) {
      array_push($this->allkids, $s);
      if($s->count_kids())
	$this->fill_allkids($s->get_kids());
    }
  }
  
  public function get_allkids() {
    // return all kids of a snip
    if(!$this->allkids && $this->count_kids())
      $this->fill_allkids($this->get_kids());
    return $this->allkids;
  }
  
  public function get_tags($name) {
    // return all specified tags of sub-snips
    $tags = array();
    foreach($this->get_allkids() as $s) {
      if($s->get_tag() == $name)
	$tags[] = $s;
    }
    return $tags;
  }

  public function __toString() {
    // magic method returns debug-info
    return sprintf("%6d %s", $this->get_beg(), $this->tree());
  }

  public function get_next() {
    // return following snip
    $mom = $this->get_mom();
    if(!$mom)
      return false;

    foreach($bros = $mom->get_kids() as $k => $s)
      if($s == $this)
	return $bros[$k+1];

    return false;
  }
  
} // EOclass ml_snippet


class ml_parser {

  public
    $snips = array(),	// all doc
    $map = array(),	// 
    $EOpubs;

  protected
    $doc = null,	// copy of doc
    $doctype = null,	// XML, HTML
    $len = null,	// doclen
    $EOprots;

  public static function eo_tag($doc, $p1) { return strpos($doc, '>', $p1); }
  // public static function bo_tag($doc, $p2) { return strrpos($doc, '<', $p2); }
  public static function bo_tag($doc, $p2) {
    if($p2 >= strlen($doc)) return false;
    for($i = $p2; $i>=0; $i--)
      if($doc[$i] == '<')
	return $i;
    return false;
  }

  public function __construct($doc, $file=false, $init=true) {
    if($file) {
      if(!file_exists($doc)) {
	echo "404 $doc".LF;
	return null;
      }
      $doc = file_get_contents($doc);
    }
    $this->doc = $doc;
    $this->len = strlen($this->doc);
    if($init) {
      $this->parse();
      $this->init_map();
      $this->genealogy();
    }
  }
  public function is_text($p) { return !$this->in_hole($p); }
  public function in_hole($p) { return false; }

  protected function add_snippet($p1, $tag, $type=0) {
    $snip = new ml_snippet($this->doc, $p1, $tag, $type);
    $this->snips[] = $snip;
    return $snip;
  }

  protected function is_endtag($p1) { return ($this->doc[$p1+1] == '/'); }
  protected function is_empty($p2)  { return ($this->doc[$p2-1] == '/'); }
  
  public function parse($bp=0, $ep=false) {
    // recursive block-parser
    if(!$ep) $ep = $this->len; //strlen($this->doc);

    if($bp === false || $bp >= $ep) return $ep;
      
    $p1 = strpos($this->doc, '<', $bp);
    if($p1 === false) return $ep;

    // doctype, xml
    if($bp == 0 && (strpos($this->doc, '<!') === $p1
		    || strpos($this->doc, '<'.'?') === $p1
		    )
       ) {
      $snip = $this->add_snippet($p1, false);
      $p2 = strcspn($this->doc, ' />', $p1);
      $snip->set_tag(substr($this->doc, $p1+2, $p2 - $p1-2));
      $p2 = $this->eo_tag($this->doc, $p1);
      $snip->set_end($p2, true);

      switch($snip->get_tag()) {
      case 'DOCTYPE': $this->doctype = 'HTML'; break;
      case 'XML':     $this->doctype = 'XML'; break;
      }
      if(DEBUG) echo "doctype {$this->doctype}".LF;
      return $this->parse($p2, $ep);
    }
    
    // skip remarks
    if(substr($this->doc, $p1, 4) == '<!--') {
      $snip = $this->add_snippet($p1, '<!--', ml_snippet::SnipREMARK);
      $p2 = strpos($this->doc, '-->', $p1+4);
      if($p2) $p2 += 2;
      $snip->set_end($p2, true);
      return $this->parse($p2, $ep);
    }
    
    $p2 = $this->eo_tag($this->doc, $p1);
    if($p2 === false) return $ep;

    if($this->is_endtag($p1)) return $p2;
    
    $snip = $this->add_snippet($p1, true);
    if($this->is_empty($p2) || !$snip->is_cont($this->doctype)) {
      $snip->set_end($p2, true);
      if(DEBUG) echo "empty ". $snip->get_tag() .LF;
      return $this->parse($p2, $ep);
    }

    // skip holes
    if($snip->is_hole()) {
      $p2 = stripos($this->doc, "</{$snip->get_tag()}", $p2);
      $p2 = ml_parser::eo_tag($this->doc, $p2);
      $snip->set_end($p2, true);
      return $this->parse($p2, $ep);
    }

    // recursive parse kids
    $p2 = $this->parse($p2, $ep);
    $snip->set_end($p2);

    // parse bros
    return $this->parse($p2, $ep);
  }

  protected function init_map() {
    foreach($this->snips as $snip)
      $this->map[] = array('bp' => $snip->get_beg(), 'snip' => $snip);
  }
  
  public function find_snip($bp) {
    // binary search over the map
    $n = count($this->map);
    for($x1 = 0, $x2 = $n; $x1+1 < $x2;) {
      $x = floor(($x1 + $x2) / 2);
      if($this->map[$x]['bp'] == $bp) { $x1 = $x; break; }
      if($this->map[$x]['bp'] < $bp) $x1 = $x;
      if($this->map[$x]['bp'] > $bp) $x2 = $x;
    }
    // return snip with lower beg and higher end
    for(; $x1 > 0 && $this->map[$x1]['snip']->get_end() < $bp; $x1--);
    return $x1 > 0 ? $this->map[$x1]['snip'] : null;
  }
  
  protected function genealogy() {
    foreach($this->snips as $snip) {
      $mom = $this->find_snip($snip->get_beg()-1);
      if($mom && $snip->get_end() >= $mom->get_end())
	$mom = $mom->get_mom();
      if($mom) {
	$snip->set_mom($mom);
	$mom->add_kid($snip);
      }
    }
  }

  public function info() {
    // put together some doc-info
    $info = new stdclass;
    $info->doctype = $this->doctype;
    $info->len = $this->len;
    $info->allkids = count($this->snips);
    return $info;
  }
  
  public function get_tags($name, $attrs=null, $set0=null) {
    // return all specified tags of a doc
    if(!is_array($set0)) $set0 = $this->snips;
    $tags = array();
    foreach($set0 as $s) {
      if($s->get_tag() == $name) {
	if(is_array($attrs)) {
	  $t = $s->get_attrs();
	  foreach($attrs as $a1 => $v) {
	    if(!isset($t[$a1])) continue 2;
	    if($v !== null && $t[$a1] != $v) continue 2;
	  }
	}
	$tags[] = $s;
      }
    }
    return $tags;
  }
  
} // EOclass ml_parser
